@@ -78,7 +78,6 @@ struct _GdmSessionDirectPrivate
 
         GdmSessionWorkerJob *job;
         GPid                 session_pid;
-        guint32              is_authenticated : 1;
         guint32              is_running : 1;
 
         /* object lifetime scope */
@@ -335,7 +334,6 @@ gdm_session_direct_handle_authenticated (GdmSessionDirect *session,
         dbus_connection_send (connection, reply, NULL);
         dbus_message_unref (reply);
 
-        session->priv->is_authenticated = TRUE;
         _gdm_session_authenticated (GDM_SESSION (session));
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -363,7 +361,6 @@ gdm_session_direct_handle_authentication_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'authentication-failed' signal");
 
-        session->priv->is_authenticated = FALSE;
         _gdm_session_authentication_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -1538,13 +1535,6 @@ gdm_session_direct_init (GdmSessionDirect *session)
 
 }
 
-static void
-worker_stopped (GdmSessionWorkerJob *job,
-                GdmSessionDirect    *session)
-{
-        g_debug ("GdmSessionDirect: Worker job stopped");
-}
-
 static void
 worker_started (GdmSessionWorkerJob *job,
                 GdmSessionDirect    *session)
@@ -1559,9 +1549,7 @@ worker_exited (GdmSessionWorkerJob *job,
 {
         g_debug ("GdmSessionDirect: Worker job exited: %d", code);
 
-        if (!session->priv->is_authenticated) {
-                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
-        } else if (session->priv->is_running) {
+        if (session->priv->is_running) {
                 _gdm_session_session_exited (GDM_SESSION (session), code);
         }
 }
@@ -1573,9 +1561,7 @@ worker_died (GdmSessionWorkerJob *job,
 {
         g_debug ("GdmSessionDirect: Worker job died: %d", signum);
 
-        if (!session->priv->is_authenticated) {
-                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
-        } else if (session->priv->is_running) {
+        if (session->priv->is_running) {
                 _gdm_session_session_died (GDM_SESSION (session), signum);
         }
 }
@@ -1587,10 +1573,6 @@ start_worker (GdmSessionDirect *session)
 
         session->priv->job = gdm_session_worker_job_new ();
         gdm_session_worker_job_set_server_address (session->priv->job, session->priv->server_address);
-        g_signal_connect (session->priv->job,
-                          "stopped",
-                          G_CALLBACK (worker_stopped),
-                          session);
         g_signal_connect (session->priv->job,
                           "started",
                           G_CALLBACK (worker_started),
@@ -1612,9 +1594,6 @@ start_worker (GdmSessionDirect *session)
 static void
 stop_worker (GdmSessionDirect *session)
 {
-        g_signal_handlers_disconnect_by_func (session->priv->job,
-                                              G_CALLBACK (worker_stopped),
-                                              session);
         g_signal_handlers_disconnect_by_func (session->priv->job,
                                               G_CALLBACK (worker_started),
                                               session);
@@ -2051,7 +2030,6 @@ gdm_session_direct_close (GdmSession *session)
         g_hash_table_remove_all (impl->priv->environment);
 
         impl->priv->session_pid = -1;
-        impl->priv->is_authenticated = FALSE;
         impl->priv->is_running = FALSE;
 }
 
