@@ -281,6 +281,8 @@ void
 gdm_option_widget_set_default_item (GdmOptionWidget *widget,
                                     const char      *item)
 {
+        char *active;
+
         g_return_if_fail (GDM_IS_OPTION_WIDGET (widget));
         g_return_if_fail (item == NULL ||
                           gdm_option_widget_lookup_item (widget, item,
@@ -299,6 +301,31 @@ gdm_option_widget_set_default_item (GdmOptionWidget *widget,
                 widget->priv->default_item_id = g_strdup (item);
 
                 g_object_notify (G_OBJECT (widget), "default-item");
+
+        } 
+
+        /* If a row has already been selected, then reset the selection to
+         * the active row.  This way when a user fails to authenticate, any
+         * previously selected value will still be selected.
+         */
+        active = gdm_option_widget_get_active_item (widget);
+
+        if (active != NULL && item != NULL &&
+            strcmp (gdm_option_widget_get_active_item (widget),
+                    item) != 0) {
+                GtkTreeRowReference *row;
+                GtkTreePath         *active_path;
+                GtkTreeModel        *model;
+
+                gdm_option_widget_set_active_item (widget, active);
+                active_path = gtk_tree_row_reference_get_path (widget->priv->active_row);
+                model = GTK_TREE_MODEL (widget->priv->list_store);
+                if (active_path != NULL) {
+                        row = gtk_tree_row_reference_new (model, active_path);
+                        activate_from_row (widget, row);
+                        gtk_tree_path_free (active_path);
+                        gtk_tree_row_reference_free (row);
+                }
         }
 }
 
@@ -509,7 +536,7 @@ gdm_option_widget_class_init (GdmOptionWidgetClass *klass)
                                          PROP_DEFAULT_ITEM,
                                          g_param_spec_string ("default-item",
                                                               _("Default Item"),
-                                                              _("The id of the default item"),
+                                                              _("The ID of the default item"),
                                                               NULL,
                                                               G_PARAM_READWRITE));
 
@@ -618,9 +645,10 @@ path_is_separator (GdmOptionWidget *widget,
 static gboolean
 gdm_option_widget_check_visibility (GdmOptionWidget *widget)
 {
-        if (widget->priv->number_of_top_rows > 0 ||
-            widget->priv->number_of_middle_rows > 1 || 
-            widget->priv->number_of_bottom_rows > 0) {
+        if ((widget->priv->number_of_middle_rows != 0) &&
+            (widget->priv->number_of_top_rows > 0 ||
+             widget->priv->number_of_middle_rows > 1 || 
+             widget->priv->number_of_bottom_rows > 0)) {
                 gtk_widget_show (widget->priv->items_combo_box);
                 gtk_widget_show (widget->priv->label);
 
