@@ -46,6 +46,7 @@
 
 #include "gdm-user-manager.h"
 #include "gdm-user-private.h"
+#include "gdm-settings-keys.h"
 
 #define GDM_USER_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_USER_MANAGER, GdmUserManagerPrivate))
 
@@ -76,34 +77,11 @@
 
 #define DEFAULT_GLOBAL_FACE_DIR DATADIR "/faces"
 #define DEFAULT_USER_ICON       "stock_person"
-#define DEFAULT_EXCLUDE         { "bin",        \
-                                  "root",       \
-                                  "daemon",     \
-                                  "adm",        \
-                                  "lp",         \
-                                  "sync",       \
-                                  "shutdown",   \
-                                  "halt",       \
-                                  "mail",       \
-                                  "news",       \
-                                  "uucp",       \
-                                  "operator",   \
-                                  "nobody",     \
-                                  "nobody4",    \
-                                  "noaccess",   \
-                                  GDM_USERNAME, \
-                                  "postgres",   \
-                                  "pvm",        \
-                                  "rpm",        \
-                                  "nfsnobody",  \
-                                  "pcap",       \
-                                  NULL }
 
 struct GdmUserManagerPrivate
 {
         GHashTable            *users;
         GHashTable            *sessions;
-        GHashTable            *exclusions;
         GHashTable            *shells;
         DBusGConnection       *connection;
         DBusGProxy            *seat_proxy;
@@ -112,6 +90,10 @@ struct GdmUserManagerPrivate
         GFileMonitor          *passwd_monitor;
         GFileMonitor          *shells_monitor;
 
+        GSList                *exclude;
+        GSList                *include;
+        gboolean               include_all;
+
         guint                  reload_id;
         guint                  ck_history_id;
 
@@ -157,8 +139,12 @@ start_new_login_session (GdmUserManager *manager)
 
         res = g_spawn_command_line_async ("gdmflexiserver -s", &error);
         if (! res) {
-                g_warning ("Unable to start new login: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Unable to start new login: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Unable to start new login");
+                }
         }
 
         return res;
@@ -177,7 +163,7 @@ _get_primary_user_session_id (GdmUserManager *manager,
         char       *primary_ssid;
 
         if (manager->priv->seat_id == NULL || manager->priv->seat_id[0] == '\0') {
-                g_debug ("GdmUserManager: display seat id is not set; can't switch sessions");
+                g_debug ("GdmUserManager: display seat ID is not set; can't switch sessions");
                 return NULL;
         }
 
@@ -194,9 +180,13 @@ _get_primary_user_session_id (GdmUserManager *manager,
                                  G_TYPE_BOOLEAN, &can_activate_sessions,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_warning ("unable to determine if seat can activate sessions: %s",
-                           error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("unable to determine if seat can activate sessions: %s",
+                                   error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("unable to determine if seat can activate sessions");
+                }
                 goto out;
         }
 
@@ -313,8 +303,12 @@ session_is_login_window (GdmUserManager *manager,
                                  G_TYPE_STRING, &session_type,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_debug ("Failed to identify the session type: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_debug ("Failed to identify the session type: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_debug ("Failed to identify the session type");
+                }
                 goto out;
         }
 
@@ -343,7 +337,7 @@ _get_login_window_session_id (GdmUserManager *manager)
         int         i;
 
         if (manager->priv->seat_id == NULL || manager->priv->seat_id[0] == '\0') {
-                g_debug ("GdmUserManager: display seat id is not set; can't switch sessions");
+                g_debug ("GdmUserManager: display seat ID is not set; can't switch sessions");
                 return NULL;
         }
 
@@ -360,9 +354,13 @@ _get_login_window_session_id (GdmUserManager *manager)
                                  G_TYPE_BOOLEAN, &can_activate_sessions,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_warning ("unable to determine if seat can activate sessions: %s",
-                           error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("unable to determine if seat can activate sessions: %s",
+                                   error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("unable to determine if seat can activate sessions");
+                }
                 goto out;
         }
 
@@ -379,9 +377,13 @@ _get_login_window_session_id (GdmUserManager *manager)
                                  dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH), &sessions,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_warning ("unable to determine sessions for user: %s",
-                           error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("unable to determine sessions for user: %s",
+                                   error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("unable to determine sessions for user");
+                }
                 goto out;
         }
 
@@ -520,8 +522,12 @@ get_seat_id_for_session (DBusGConnection *connection,
                                  DBUS_TYPE_G_OBJECT_PATH, &seat_id,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_debug ("Failed to identify the current seat: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_debug ("Failed to identify the current seat: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_debug ("Failed to identify the current seat");
+                }
         }
  out:
         if (proxy != NULL) {
@@ -560,8 +566,12 @@ get_x11_display_for_session (DBusGConnection *connection,
                                  G_TYPE_STRING, &x11_display,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_debug ("Failed to identify the x11 display: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_debug ("Failed to identify the x11 display: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_debug ("Failed to identify the x11 display");
+                }
         }
  out:
         if (proxy != NULL) {
@@ -571,6 +581,43 @@ get_x11_display_for_session (DBusGConnection *connection,
         return x11_display;
 }
 
+static gint
+match_name_cmpfunc (gconstpointer a,
+                    gconstpointer b)
+{
+        if (a == NULL || b == NULL) {
+                return -1;
+        }
+
+        return g_strcmp0 ((char *) a,
+                          (char *) b);
+}
+
+static gboolean
+user_in_exclude_list (GdmUserManager *manager,
+                      const char *user)
+{
+        GSList   *found;
+        gboolean  ret = FALSE;
+
+        g_debug ("checking exclude list");
+
+        /* always exclude the "gdm" user. */
+        if (user == NULL || (strcmp (user, GDM_USERNAME) == 0)) {
+                return TRUE;
+        }
+
+        if (manager->priv->exclude != NULL) {
+                found = g_slist_find_custom (manager->priv->exclude,
+                                             user,
+                                             match_name_cmpfunc);
+                if (found != NULL) {
+                        ret = TRUE;
+                }
+        }
+        return ret;
+}
+
 static gboolean
 maybe_add_session_for_user (GdmUserManager *manager,
                             GdmUser        *user,
@@ -600,7 +647,7 @@ maybe_add_session_for_user (GdmUserManager *manager,
                 goto out;
         }
 
-        if (g_hash_table_lookup (manager->priv->exclusions, gdm_user_get_user_name (user))) {
+        if (user_in_exclude_list (manager, gdm_user_get_user_name (user))) {
                 g_debug ("GdmUserManager: excluding user '%s'", gdm_user_get_user_name (user));
                 goto out;
         }
@@ -655,8 +702,12 @@ add_sessions_for_user (GdmUserManager *manager,
                                  &sessions,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_debug ("Failed to find sessions for user: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_debug ("Failed to find sessions for user: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_debug ("Failed to find sessions for user");
+                }
                 goto out;
         }
 
@@ -782,8 +833,12 @@ get_current_seat_id (DBusGConnection *connection)
                                  &session_id,
                                  G_TYPE_INVALID);
         if (! res) {
-                g_debug ("Failed to identify the current session: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_debug ("Failed to identify the current session: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_debug ("Failed to identify the current session");
+                }
                 goto out;
         }
 
@@ -827,8 +882,12 @@ get_uid_from_session_id (GdmUserManager *manager,
         g_object_unref (proxy);
 
         if (! res) {
-                g_warning ("Failed to query the session: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Failed to query the session: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Failed to query the session");
+                }
                 return FALSE;
         }
 
@@ -861,12 +920,12 @@ seat_session_added (DBusGProxy     *seat_proxy,
         errno = 0;
         pwent = getpwuid (uid);
         if (pwent == NULL) {
-                g_warning ("Unable to lookup user id %d: %s", (int)uid, g_strerror (errno));
+                g_warning ("Unable to lookup user ID %d: %s", (int)uid, g_strerror (errno));
                 return;
         }
 
         /* check exclusions up front */
-        if (g_hash_table_lookup (manager->priv->exclusions, pwent->pw_name)) {
+        if (user_in_exclude_list (manager, pwent->pw_name)) {
                 g_debug ("GdmUserManager: excluding user '%s'", pwent->pw_name);
                 return;
         }
@@ -942,8 +1001,12 @@ get_seat_proxy (GdmUserManager *manager)
         error = NULL;
         manager->priv->connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
         if (manager->priv->connection == NULL) {
-                g_warning ("Failed to connect to the D-Bus daemon: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Failed to connect to the D-Bus daemon: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Failed to connect to the D-Bus daemon");
+                }
                 return;
         }
 
@@ -962,8 +1025,13 @@ get_seat_proxy (GdmUserManager *manager)
                                                  &error);
 
         if (proxy == NULL) {
-                g_warning ("Failed to connect to the ConsoleKit seat object: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Failed to connect to the ConsoleKit seat object: %s",
+                                   error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Failed to connect to the ConsoleKit seat object");
+                }
                 return;
         }
 
@@ -1115,7 +1183,11 @@ parse_ck_history_line (const char *line,
         error = NULL;
         re = g_regex_new ("(?P<username>[0-9a-zA-Z]+)[ ]+(?P<frequency>[0-9]+)", 0, 0, &error);
         if (re == NULL) {
-                g_critical ("%s", error->message);
+                if (error != NULL) {
+                        g_critical ("%s", error->message);
+                } else {
+                        g_critical ("Error in regex call");
+                }
                 goto out;
         }
 
@@ -1169,7 +1241,7 @@ process_ck_history_line (GdmUserManager *manager,
                 return;
         }
 
-        if (g_hash_table_lookup (manager->priv->exclusions, username)) {
+        if (user_in_exclude_list (manager, username)) {
                 g_debug ("GdmUserManager: excluding user '%s'", username);
                 g_free (username);
                 return;
@@ -1258,8 +1330,12 @@ reload_ck_history (GdmUserManager *manager)
         g_debug ("GdmUserManager: running '%s'", command);
         error = NULL;
         if (! g_shell_parse_argv (command, NULL, &argv, &error)) {
-                g_warning ("Could not parse command: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Could not parse command: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Could not parse command");
+                }
                 goto out;
         }
 
@@ -1277,8 +1353,12 @@ reload_ck_history (GdmUserManager *manager)
                                         &error);
         g_strfreev (argv);
         if (! res) {
-                g_warning ("Unable to run ck-history: %s", error->message);
-                g_error_free (error);
+                if (error != NULL) {
+                        g_warning ("Unable to run ck-history: %s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Unable to run ck-history");
+                }
                 goto out;
         }
 
@@ -1297,6 +1377,35 @@ reload_ck_history (GdmUserManager *manager)
         g_free (command);
 }
 
+static void
+add_included_user (char *username, GdmUserManager *manager)
+{
+        GdmUser     *user;
+
+        g_debug ("Adding included user %s", username);
+        /*
+         * The call to gdm_user_manager_get_user will add the user if it is
+         * valid and not already in the hash.
+         */
+        user = gdm_user_manager_get_user (manager, username);
+        if (user == NULL) {
+                g_debug ("GdmUserManager: unable to lookup user '%s'", username);
+                g_free (username);
+                return;
+        }
+}
+
+static void
+add_included_users (GdmUserManager *manager)
+{
+        /* Add users who are specifically included */
+        if (manager->priv->include != NULL) {
+                g_slist_foreach (manager->priv->include,
+                                 (GFunc)add_included_user,
+                                 (gpointer)manager);
+        }
+}
+
 static void
 reload_passwd (GdmUserManager *manager)
 {
@@ -1329,48 +1438,58 @@ reload_passwd (GdmUserManager *manager)
                 }
         }
 
-        for (pwent = fgetpwent (fp); pwent != NULL; pwent = fgetpwent (fp)) {
-                GdmUser *user;
+        if (manager->priv->include_all != TRUE) {
+                g_debug ("GdmUserManager: include_all is FALSE");
+        } else {
+                g_debug ("GdmUserManager: include_all is TRUE");
 
-                user = NULL;
+                for (pwent = fgetpwent (fp);
+                     pwent != NULL;
+                     pwent = fgetpwent (fp)) {
+                        GdmUser *user;
 
-                /* Skip users below MinimalUID... */
-                if (pwent->pw_uid < DEFAULT_MINIMAL_UID) {
-                        continue;
-                }
+                        user = NULL;
 
-                /* ...And users w/ invalid shells... */
-                if (pwent->pw_shell == NULL ||
-                    !g_hash_table_lookup (manager->priv->shells, pwent->pw_shell)) {
-                        g_debug ("GdmUserManager: skipping user with bad shell: %s", pwent->pw_name);
-                        continue;
-                }
+                        /* Skip users below MinimalUID... */
+                        if (pwent->pw_uid < DEFAULT_MINIMAL_UID) {
+                                continue;
+                        }
 
-                /* ...And explicitly excluded users */
-                if (g_hash_table_lookup (manager->priv->exclusions, pwent->pw_name)) {
-                        g_debug ("GdmUserManager: explicitly skipping user: %s", pwent->pw_name);
-                        continue;
-                }
+                        /* ...And users w/ invalid shells... */
+                        if (pwent->pw_shell == NULL ||
+                            !g_hash_table_lookup (manager->priv->shells,
+                                                  pwent->pw_shell)) {
+                                g_debug ("GdmUserManager: skipping user with bad shell: %s", pwent->pw_name);
+                                continue;
+                        }
 
-                user = g_hash_table_lookup (manager->priv->users, pwent->pw_name);
+                        /* ...And explicitly excluded users */
+                        if (user_in_exclude_list (manager, pwent->pw_name)) {
+                                g_debug ("GdmUserManager: explicitly skipping user: %s", pwent->pw_name);
+                                continue;
+                        }
 
-                /* Update users already in the *new* list */
-                if (g_slist_find (new_users, user)) {
+                        user = g_hash_table_lookup (manager->priv->users,
+                                                    pwent->pw_name);
+
+                        /* Update users already in the *new* list */
+                        if (g_slist_find (new_users, user)) {
+                                _gdm_user_update (user, pwent);
+                                continue;
+                        }
+
+                        if (user == NULL) {
+                                user = create_user (manager);
+                        } else {
+                                g_object_ref (user);
+                        }
+
+                        /* Freeze & update users not already in the new list */
+                        g_object_freeze_notify (G_OBJECT (user));
                         _gdm_user_update (user, pwent);
-                        continue;
+
+                        new_users = g_slist_prepend (new_users, user);
                 }
-
-                if (user == NULL) {
-                        user = create_user (manager);
-                } else {
-                        g_object_ref (user);
-                }
-
-                /* Freeze & update users not already in the new list */
-                g_object_freeze_notify (G_OBJECT (user));
-                _gdm_user_update (user, pwent);
-
-                new_users = g_slist_prepend (new_users, user);
         }
 
         /* Go through and handle removed users */
@@ -1397,6 +1516,8 @@ reload_passwd (GdmUserManager *manager)
                 }
         }
 
+        add_included_users (manager);
+
  out:
         /* Cleanup */
 
@@ -1551,77 +1672,109 @@ gdm_user_manager_class_init (GdmUserManagerClass *klass)
         g_type_class_add_private (klass, sizeof (GdmUserManagerPrivate));
 }
 
+static void
+gdm_set_string_list (char *value, GSList **retval)
+{
+        char **temp_array;
+        int    i;
+
+        *retval = NULL;
+
+        if (value == NULL || *value == '\0') {
+                g_debug ("Not adding NULL user");
+                *retval = NULL;
+                return;
+        }
+
+        temp_array = g_strsplit (value, ",", 0);
+        for (i = 0; temp_array[i] != NULL; i++) {
+                g_debug ("Adding value %s", temp_array[i]);
+                g_strstrip (temp_array[i]);
+                *retval = g_slist_prepend (*retval, g_strdup (temp_array[i]));
+        }
+
+        g_strfreev (temp_array);
+}
+
+
 static void
 gdm_user_manager_init (GdmUserManager *manager)
 {
         int            i;
         GFile         *file;
         GError        *error;
-        const char    *exclude_default[] = DEFAULT_EXCLUDE;
+        char          *temp;
+        gboolean       res;
 
         manager->priv = GDM_USER_MANAGER_GET_PRIVATE (manager);
 
+        /* exclude/include */
+        g_debug ("Setting users to include:");
+        res = gdm_settings_client_get_string  (GDM_KEY_INCLUDE,
+                                               &temp);
+        gdm_set_string_list (temp, &manager->priv->include);
+
+        g_debug ("Setting users to exclude:");
+        res = gdm_settings_client_get_string  (GDM_KEY_EXCLUDE,
+                                               &temp);
+        gdm_set_string_list (temp, &manager->priv->exclude);
+
+        res = gdm_settings_client_get_boolean (GDM_KEY_INCLUDE_ALL,
+                                               &manager->priv->include_all);
+
         /* sessions */
         manager->priv->sessions = g_hash_table_new_full (g_str_hash,
                                                          g_str_equal,
                                                          g_free,
                                                          g_free);
 
-        /* exclusions */
-        manager->priv->exclusions = g_hash_table_new_full (g_str_hash,
-                                                           g_str_equal,
-                                                           g_free,
-                                                           NULL);
-        for (i = 0; exclude_default[i] != NULL; i++) {
-                g_hash_table_insert (manager->priv->exclusions,
-                                     g_strdup (exclude_default [i]),
-                                     GUINT_TO_POINTER (TRUE));
-        }
-
-        /* /etc/shells */
-        manager->priv->shells = g_hash_table_new_full (g_str_hash,
-                                                       g_str_equal,
-                                                       g_free,
-                                                       NULL);
-        reload_shells (manager);
-        file = g_file_new_for_path (_PATH_SHELLS);
-        error = NULL;
-        manager->priv->shells_monitor = g_file_monitor_file (file,
-                                                             G_FILE_MONITOR_NONE,
-                                                             NULL,
-                                                             &error);
-        if (manager->priv->shells_monitor != NULL) {
-                g_signal_connect (manager->priv->shells_monitor,
-                                  "changed",
-                                  G_CALLBACK (on_shells_monitor_changed),
-                                  manager);
-        } else {
-                g_warning ("Unable to monitor %s: %s", _PATH_SHELLS, error->message);
-                g_error_free (error);
-        }
-        g_object_unref (file);
-
-        /* /etc/passwd */
+        /* users */
         manager->priv->users = g_hash_table_new_full (g_str_hash,
                                                       g_str_equal,
                                                       g_free,
                                                       (GDestroyNotify) g_object_run_dispose);
-        file = g_file_new_for_path (PATH_PASSWD);
-        manager->priv->passwd_monitor = g_file_monitor_file (file,
-                                                             G_FILE_MONITOR_NONE,
-                                                             NULL,
-                                                             &error);
-        if (manager->priv->passwd_monitor != NULL) {
-                g_signal_connect (manager->priv->passwd_monitor,
-                                  "changed",
-                                  G_CALLBACK (on_passwd_monitor_changed),
-                                  manager);
-        } else {
-                g_warning ("Unable to monitor %s: %s", PATH_PASSWD, error->message);
-                g_error_free (error);
-        }
-        g_object_unref (file);
 
+        if (manager->priv->include_all == TRUE) {
+                /* /etc/shells */
+                manager->priv->shells = g_hash_table_new_full (g_str_hash,
+                                                               g_str_equal,
+                                                               g_free,
+                                                               NULL);
+                reload_shells (manager);
+                file = g_file_new_for_path (_PATH_SHELLS);
+                error = NULL;
+                manager->priv->shells_monitor = g_file_monitor_file (file,
+                                                                     G_FILE_MONITOR_NONE,
+                                                                     NULL,
+                                                                     &error);
+                if (manager->priv->shells_monitor != NULL) {
+                        g_signal_connect (manager->priv->shells_monitor,
+                                          "changed",
+                                          G_CALLBACK (on_shells_monitor_changed),
+                                          manager);
+                } else {
+                        g_warning ("Unable to monitor %s: %s", _PATH_SHELLS, error->message);
+                        g_error_free (error);
+                }
+                g_object_unref (file);
+
+                /* /etc/passwd */
+                file = g_file_new_for_path (PATH_PASSWD);
+                manager->priv->passwd_monitor = g_file_monitor_file (file,
+                                                                     G_FILE_MONITOR_NONE,
+                                                                     NULL,
+                                                                     &error);
+                if (manager->priv->passwd_monitor != NULL) {
+                        g_signal_connect (manager->priv->passwd_monitor,
+                                          "changed",
+                                          G_CALLBACK (on_passwd_monitor_changed),
+                                          manager);
+                } else {
+                        g_warning ("Unable to monitor %s: %s", PATH_PASSWD, error->message);
+                        g_error_free (error);
+                }
+                g_object_unref (file);
+        }
 
         get_seat_proxy (manager);
 
@@ -1642,6 +1795,14 @@ gdm_user_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
+        if (manager->priv->exclude != NULL) {
+                g_slist_free (manager->priv->exclude);
+        }
+
+        if (manager->priv->include != NULL) {
+                g_slist_free (manager->priv->include);
+        }
+
         if (manager->priv->seat_proxy != NULL) {
                 g_object_unref (manager->priv->seat_proxy);
         }
